/* $Id: matrix.cc,v 1.2 1997/09/20 20:07:13 sean Exp sean $ */
/*
$Log: matrix.cc,v $
Revision 1.2  1997/09/20 20:07:13  sean
/dev/null

Revision 1.1  1997/09/06 23:36:48  sean
/dev/null

Revision 1.6  1997/04/12 22:32:46  sean
*** empty log message ***

Revision 1.5  1995/08/03 18:45:43  sean
optimize?  re-think pointer/object return policy

*/
#include <stdio.h>
#include <math.h>
#include "matrix.h"

v1 operator+(v1 a, v1 b) { return v1(a.x + b.x); }
v1 operator-(v1 a, v1 b) { return v1(a.x - b.x); }
v1 operator-(v1 a       ) { return v1( - a.x); }
v1 operator*(v1 a, v1 b) { return v1(a.x * b.x); }
v1 operator/(v1 a, v1 b) { return v1(a.x / b.x); }

v1::print() {
    printf("[%7.2f]",x);
}

v2 operator*(v2 a, v2 b) { return v2(a.x * b.x , a.y * b.y); }
v2 operator+(v2 a, v2 b) { return v2(a.x + b.x , a.y + b.y); }
v2 operator-(v2 a, v2 b) { return v2(a.x - b.x , a.y - b.y); }
v2 operator-(v2 a       ) { return v2( - a.x, - a.y); }

real v2::length() {
  return (real) sqrt(x * x + y * y);
};

v2 v2::unit() {
  real length, nx, ny;
  
  length = (real) sqrt(x * x + y * y);
  nx = x / length;
  ny = y / length;
  return v2(nx, ny);
}

v2 v2_dot(v2 a, v2 b) {
  real dot_product;

  dot_product = a.x * b.x + a.y * b.y;
}

v2 v2_scalar_mul(real c, v2 a) {
  return v2(c * a.x, c * a.y);
}

v2::print() {
    printf("[%7.2f %7.2f]",x,y);
}

v3 operator*(v3 a, v3 b) { 
  return v3(a.x * b.x , a.y * b.y, a.z * b.z); }
v3 operator+(v3 a, v3 b) { 
  return v3(a.x + b.x , a.y + b.y, a.z + b.z); }
v3 operator-(v3 a, v3 b) {
  return v3(a.x - b.x , a.y - b.y, a.z - b.z); }
v3 operator-(v3 a       ) {
  return v3( - a.x ,  - a.y,  - a.z); }

real v3::length() {
  return (real) sqrt(x * x + y * y + z * z);
};

v3 v3::unit() {
  real length, nx, ny, nz;
  
  length = (real) sqrt(x * x + y * y + z * z);
  nx = x / length;
  ny = y / length;
  nz = z / length;
  return v3(nx, ny, nz);
}
v3 v3::positive() {
  real xt, yt, zt;
  xt = (x > 0.0) ? x : 0.0;
  yt = (y > 0.0) ? y : 0.0;
  zt = (z > 0.0) ? z : 0.0;
  return v3(xt, yt, zt);
}

real v3_dot(v3 a, v3 b) {
  return a.x * b.x + a.y * b.y + a.z * b.z;
}

v3 v3_cross(v3 a, v3 b) {
  return v3((a.y * b.z - a.z * b.y), 
	    (a.z * b.x - a.x * b.z),
	    (a.x * b.y - a.y * b.x));
}

v3 v3_scalar_mul(real c, v3 a) {
  return v3(c * a.x, c * a.y, c * a.z);
}

v3::print() {
    printf("[%7.2f %7.2f %7.2f]",
	   x,y,z);
}

m2 operator-(m2 a) {
  return m2(- a.e11, - a.e12 , - a.e21, - a.e22);
}

m2 operator*(m2 a, m2 b) {
  return m2(a.e11 * b.e11 + a.e12 * b.e21,
	    a.e11 * b.e12 + a.e12 * b.e22,
	    a.e21 * b.e11 + a.e22 * b.e21,
	    a.e21 * b.e12 + a.e22 * b.e22);
}

v2 operator*(m2 a, v2 b) {
  return v2(a.e11 * b.x + a.e12 * b.y, a.e21 * b.x + a.e22 * b.y);
}

m2 m2_scalar_mul(real c, m2 a) {
  return m2(c * a.e11, c * a.e12 , c * a.e21, c * a.e22);
}

real m2::det() {
  return e11 * e22 - e12 * e21;
}

m2 m2::inv() {
  real d;

  d = det();
  if (d != 0.0)
    return m2(e22 / d, - e12 / d, - e21 / d, e11 / d);
  return m2(0.0,0.0,0.0,0.0);
}

m2::print() {
    printf("/%7.2f %7.2f\\\n\\%7.2f %7.2f/",
   e11,e12,
   e21,e22);
}


m3 operator-(m3 t) { 
  return m3(- t.e11, - t.e12, - t.e13, - t.e21, - t.e22, - t.e23,
	    - t.e31, - t.e32, - t.e33);
}

m3 operator*(m3 a, m3 b) {
  real t11, t12, t13, t21, t22, t23, t31, t32, t33;


  t11 = a.e11 * b.e11 + a.e12 * b.e21 + a.e13 * b.e31;
  t12 = a.e11 * b.e12 + a.e12 * b.e22 + a.e13 * b.e32;
  t13 = a.e11 * b.e13 + a.e12 * b.e23 + a.e13 * b.e33;

  t21 = a.e21 * b.e11 + a.e22 * b.e21 + a.e23 * b.e31;
  t22 = a.e21 * b.e12 + a.e22 * b.e22 + a.e23 * b.e32;
  t23 = a.e21 * b.e13 + a.e22 * b.e23 + a.e23 * b.e33;

  t31 = a.e31 * b.e11 + a.e32 * b.e21 + a.e33 * b.e31;
  t32 = a.e31 * b.e12 + a.e32 * b.e22 + a.e33 * b.e32;
  t33 = a.e31 * b.e13 + a.e32 * b.e23 + a.e33 * b.e33;
  return m3(t11, t12, t13, t21, t22, t23, t31, t32, t33);
}

v3 operator*(m3 a, v3 b) {
  real tx, ty, tz;


  tx = a.e11 * b.x + a.e12 * b.y + a.e13 * b.z;
  ty = a.e21 * b.x + a.e22 * b.y + a.e23 * b.z;
  tz = a.e31 * b.x + a.e32 * b.y + a.e33 * b.z;
  return v3(tx, ty, tz);
}

m3 m3_scalar_mul (real c, m3 t) { 
  return m3(c * t.e11, c * t.e12, c * t.e13, c * t.e21, c * t.e22, c * t.e23,
	    c * t.e31, c * t.e32, c * t.e33);
}

m3 m3::cofactors() {
  return m3(
	e22 * e33 - e32 *e23,
	- (e21 * e33 - e31 * e23),
	e21 * e32 - e31 * e22,

	- (e12 * e33 - e32 * e13),
	e11 * e33 - e31 * e13,
	- (e11 * e32 - e31 * e12),
	
	e12 * e23 - e22 * e13,
	- (e11 * e23 - e21 * e13),
	e11 * e22 - e21 * e12 );
}

m3 m3::transpose() {
   return m3(
	e11, e21, e31,
	e12, e22, e32,
	e13, e23, e33);
}
	
real m3::det() {
  return 
	e11 * e22 * e33 +
	e12 * e23 * e31 +
	e13 * e21 * e32 +
      - e13 * e22 * e31 -
	e12 * e21 * e33 -
	e11 * e23 * e32;
}

m3 m3::adj() {
	return cofactors().transpose();
}

m3 m3::inv() {
  real d;

  d = det();
  if (d != 0.0)
    return m3_scalar_mul(1/d, adj());
  return m3(0.0,0.0,0.0,
	    0.0,0.0,0.0,
	    0.0,0.0,0.0);
}

  m3::print() {
    printf("/%7.2f %7.2f %7.2f\\\n|%7.2f %7.2f %7.2f|\n\\%7.2f %7.2f %7.2f/",
   e11,e12,e13,
   e21,e22,e23,
   e31,e32,e33);
}

m4 operator-(m4 t) {
  return m4( - t.e11, - t.e12, - t.e13, - t.e14, 
	     - t.e21, - t.e22, - t.e23, - t.e24,
	     - t.e31, - t.e32, - t.e33, - t.e34,
	     - t.e41, - t.e42, - t.e43, - t.e44);
}

m4 operator*(m4 a, m4 b) {

  real t11, t12, t13, t14, t21, t22, t23, t24, t31, t32, t33, t34, 
       t41, t42, t43, t44;

t11 = a.e11 * b.e11 + a.e12 * b.e21 + a.e13 * b.e31 + a.e14 * b.e41;
t12 = a.e11 * b.e12 + a.e12 * b.e22 + a.e13 * b.e32 + a.e14 * b.e42; 
t13 = a.e11 * b.e13 + a.e12 * b.e23 + a.e13 * b.e33 + a.e14 * b.e43;
t14 = a.e11 * b.e14 + a.e12 * b.e24 + a.e13 * b.e34 + a.e14 * b.e44;

t21 = a.e21 * b.e11 + a.e22 * b.e21 + a.e23 * b.e31 + a.e24 * b.e41;
t22 = a.e21 * b.e12 + a.e22 * b.e22 + a.e23 * b.e32 + a.e24 * b.e42; 
t23 = a.e21 * b.e13 + a.e22 * b.e23 + a.e23 * b.e33 + a.e24 * b.e43;
t24 = a.e21 * b.e14 + a.e22 * b.e24 + a.e23 * b.e34 + a.e24 * b.e44;

t31 = a.e31 * b.e11 + a.e32 * b.e21 + a.e33 * b.e31 + a.e34 * b.e41;
t32 = a.e31 * b.e12 + a.e32 * b.e22 + a.e33 * b.e32 + a.e34 * b.e42; 
t33 = a.e31 * b.e13 + a.e32 * b.e23 + a.e33 * b.e33 + a.e34 * b.e43;
t34 = a.e31 * b.e14 + a.e32 * b.e23 + a.e33 * b.e34 + a.e34 * b.e44;

t41 = a.e41 * b.e11 + a.e42 * b.e21 + a.e43 * b.e31 + a.e44 * b.e41;
t42 = a.e41 * b.e12 + a.e42 * b.e22 + a.e43 * b.e32 + a.e44 * b.e42; 
t43 = a.e41 * b.e13 + a.e42 * b.e23 + a.e43 * b.e33 + a.e44 * b.e43;
t44 = a.e41 * b.e14 + a.e42 * b.e24 + a.e43 * b.e34 + a.e44 * b.e44;

return m4(t11, t12, t13, t14, 
	  t21, t22, t23, t24, 
	  t31, t32, t33, t34, 
	  t41, t42, t43, t44);
}

m4 m4_scalar_mul(real c, m4 t) {
  return m4( c * t.e11, c * t.e12, c * t.e13, c * t.e14, 
	     c * t.e21, c * t.e22, c * t.e23, c * t.e24,
	     c * t.e31, c * t.e32, c * t.e33, c * t.e34,
	     c * t.e41, c * t.e42, c * t.e43, c * t.e44);
}

m4::print() {
  printf("/%7.2f %7.2f %7.2f %7.2f\\\n|%7.2f %7.2f %7.2f %7.2f|\n|%7.2f %7.2f %7.2f %7.2f|\n\\%7.2f %7.2f %7.2f %7.2f/",
   e11,e12,e13,e14,
   e21,e22,e23,e24,
   e31,e32,e33,e34,
   e41,e42,e43,e44);
}
